// Global JS application module for Hawat.
var Hawat = (function () {

    function _build_param_builder(skeleton, rules) {
        //var _skeleton = Object.assign({}, skeleton);
        var _skeleton = skeleton;
        var _rules = rules;
        return function(value) {
            //var _result = Object.assign({}, _skeleton);
            var _result = _skeleton;
            _rules.forEach(function(r) {
                _result[r[0]] = value;
            });
            return _result;
        }
    }

    function _get_aod(url, param, elem, ident, snippets) {
        // Append element for query result and populate it with AJAX spinner.
        $(elem).append(
            '<div class="query-result ' + ident + '"><i class="fas fa-cog fa-spin" data-toggle="tooltip" title="' + ident + '"></i></div>'
        );
        // Obtain reference for recently created element for query result.
        elem = $(elem).children('.query-result.' + ident);

        // Perform asynchronous request.
        console.log("Request URL" + url + ", snippets: " + snippets);
        var jqxhr = $.get(url)
        .done(function(data) {
            if (data.search_result && ((Array.isArray(data.search_result) && data.search_result.length > 0) || !Array.isArray(data.search_result))) {
                console.log("Result for " + ident + " query for: " + param);
                console.debug(data);
                $(elem).children().tooltip('destroy');
                $(elem).empty();
                snippets.forEach(function(snippet) {
                    $(elem).append(data.snippets[snippet]);
                });
            }
            else {
                console.log("Empty result for " + ident + " query for: " + param);
                console.debug(data);
                // Either insert information about empty query result.
                //$(elem).html(
                //    '<i class="fas fa-check" data-toggle="tooltip" title="Empty result for ' + ident + ' query for ' + param + '"></i>'
                //);
                // Or remove the result element to reduce display clutter.
                $(elem).children().tooltip('destroy');
                $(elem).remove();
            }
        })
        .fail(function(data) {
            console.log("Failed " + ident + " query for: " + param);
            console.debug(data);
            // Either insert information about error query result.
            $(elem).html(
                '<i class="fas fa-times" data-toggle="tooltip" title="Error for ' + ident + ' query for ' + param + '"></i>'
            );
            // Or remove the result element to reduce display clutter.
            //$(elem).children().tooltip('destroy');
            //$(elem).remove();
        })
        .always(function() {
            //console.log("Finished " + ident + " query for: " + param);
        });
    }

    var _configs = {
        'APPLICATION_ROOT': '{{ hawat_current_app.config['APPLICATION_ROOT'] }}'
    };

    var _csag = {
{%- for csag_name in hawat_current_app.csag.keys() | sort %}
        '{{ csag_name }}': [
    {%- for csag in hawat_current_app.csag[csag_name] %}
        {%- if 'view' in csag %}
            {
                'title':    '{{ _(csag.title, name = '{name}') }}',
                'endpoint': '{{ csag.view.get_view_endpoint() }}',
                'icon':     '{{ get_icon(csag.view.get_menu_icon()) }}',
                'params':   _build_param_builder(
                    {{ csag.params.skeleton | tojson | safe }},
                    {{ csag.params.rules | tojson | safe }}
                )
            }{%- if not loop.last %},{%- endif %}
        {%- endif %}
    {%- endfor %}
        ]{%- if not loop.last %},{%- endif %}
{%- endfor %}
    };

    var _aods = {
{%- for aods_name in hawat_current_app.aods.keys() | sort %}
        '{{ aods_name }}': [
    {%- for aods in hawat_current_app.aods[aods_name] %}
        {%- if 'view' in aods %}
            {
                'endpoint': '{{ aods.view.get_view_endpoint() }}',
                'ident': '{{ aods.view.module_name | upper }}',
                'snippets': {{ aods.snippets | tojson | safe }},
                'params':   _build_param_builder(
                    {{ aods.params.skeleton | tojson | safe }},
                    {{ aods.params.rules | tojson | safe }}
                )
            }{%- if not loop.last %},{%- endif %}
        {%- endif %}
    {%- endfor %}
        ]{%- if not loop.last %},{%- endif %}
{%- endfor %}
    };

    return {
        get_all_csags: function() {
            return _csag;
        },

        get_all_aodss: function() {
            return _aods;
        },

        get_csag: function(name) {
            try {
                return _csag[name];
            }
            catch (err) {
                return null
            }
        },

        get_aods: function(name) {
            try {
                return _aods[name];
            }
            catch (err) {
                return null
            }
        },

        fetch_aods: function(index, elem) {
            obj_type = $(elem).data('object-type');
            obj_name = $(elem).data('object-name');
            aods     = Hawat.get_aods(obj_type);
            console.log("Fetching additional data for object '" + obj_type + " -- " + obj_name + "': " + aods.reduce(function(sum, item) { return sum + ' ' + item.endpoint }, ''));
            console.debug(aods);

            aods.forEach(function(aods_item) {
                _get_aod(
                    Flask.url_for(aods_item.endpoint, aods_item.params(obj_name)),
                    obj_name,
                    elem,
                    aods_item.ident,
                    aods_item.snippets
                );
            });
        }
    };
})();

$(function() {
    $(".object-additional-data.onload").each(Hawat.fetch_aods);
    $('.object-additional-data.ondemand').on("click", function() {
        var ref = $(this);
        ref.children().tooltip('destroy');
        ref.removeClass('ondemand');
        ref.empty();
        ref.off('click');
        Hawat.fetch_aods(0, ref);
    });
});
